<?php

declare(strict_types=1);

namespace Theme\VaporChild\Controllers;

use App\Domain\Content\Model\Content;
use App\Infrastructure\Services\Options;
use App\Infrastructure\Services\Paginator;
use App\Infrastructure\Services\UserAuth;
use Codefy\CommandBus\Exceptions\CommandPropertyNotFoundException;
use Codefy\Framework\Http\BaseController;
use Codefy\QueryBus\UnresolvableQueryHandlerException;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\NotFoundExceptionInterface;
use Psr\Http\Message\ResponseInterface;
use Psr\SimpleCache\InvalidArgumentException;
use Qubus\Config\ConfigContainer;
use Qubus\Exception\Exception;
use Qubus\Http\Factories\XmlResponseFactory;
use Qubus\Http\Session\SessionService;
use Qubus\Routing\Router;
use Qubus\View\Renderer;
use ReflectionException;

use function App\Shared\Helpers\get_all_content_with_filters;
use function App\Shared\Helpers\get_content_by;
use function App\Shared\Helpers\site_url;
use function count;
use function Qubus\Security\Helpers\t__;
use function Qubus\Support\Helpers\is_false__;

class VaporThemeController extends BaseController
{
    public function __construct(
        SessionService $sessionService,
        Router $router,
        protected ConfigContainer $configContainer,
        protected UserAuth $user,
        ?Renderer $view = null
    ) {
        parent::__construct($sessionService, $router, $view);
    }

    /**
     * @return string|null
     * @throws CommandPropertyNotFoundException
     * @throws ContainerExceptionInterface
     * @throws Exception
     * @throws InvalidArgumentException
     * @throws NotFoundExceptionInterface
     * @throws ReflectionException
     * @throws UnresolvableQueryHandlerException
     */
    public function index(): ?string
    {
        $options = Options::factory()->read(optionKey: 'vapor_theme_settings');
        $content = get_all_content_with_filters(
            contentTypeSlug: $options['vapor_content_type'] ?? '',
            status: $options['vapor_content_status'] ?? 'all',
        );
        $limit = (int) Options::factory()->read(optionKey: 'content_per_page');

        if (empty(array_filter($content))) {
            return $this->view->render(
                'theme::Vapor/views/no-content',
                ['title' => t__(msgid: '404: Not Found', domain: 'vapor-theme'), 'single' => false]
            );
        }

        return $this->view->render(
            'theme::VaporChild/views/index',
            [
                'title' => Options::factory()->read(optionKey: 'sitename'),
                'contentType' => $options['vapor_content_type'] ?? '',
                'contentLimit' => $limit,
                'contentOffset' => isset($options['vapor_content_offset']) ? (int) $options['vapor_content_offset'] : 0,
                'contentStatus' => $options['vapor_content_status'] ?? 'all',
                'paginator' => new Paginator(
                    totalItems: (int) count(array_filter($content)),
                    itemsPerPage: $limit,
                    currentPage: 0,
                ),
                'single' => false,
            ]
        );
    }

    /**
     * @param string $contentSlug
     * @return string|null
     * @throws ContainerExceptionInterface
     * @throws Exception
     * @throws InvalidArgumentException
     * @throws NotFoundExceptionInterface
     * @throws ReflectionException
     */
    public function single(string $contentSlug): ?string
    {
        /** @var Content $content */
        $content = get_content_by('slug', $contentSlug);
        if (is_false__($content)) {
            return $this->view->render(
                'theme::Vapor/views/no-content',
                ['title' => t__(msgid: '404: Not Found', domain: 'vapor-theme'), 'single' => false]
            );
        }

        return $this->view->render(
            'theme::VaporChild/views/single',
            [
                'title' => $content->title,
                'content' => $content,
                'single' => true,
            ]
        );
    }

    /**
     * @throws UnresolvableQueryHandlerException
     * @throws ReflectionException
     * @throws CommandPropertyNotFoundException
     * @throws Exception
     * @throws \Exception
     */
    public function sitemap(): ResponseInterface|string
    {
        $urls = [];
        $urls[] = ['loc' => site_url(), 'changefreq' => 'weekly', 'priority' => '1.0'];

        $posts = get_all_content_with_filters(contentTypeSlug: 'post', status: 'published');
        foreach ($posts as $post) {
            $urls[] = ['loc' => site_url($post['slug']), 'changefreq' => 'weekly', 'priority' => '0.8'];
        }

        $response = XmlResponseFactory::create(
            $this->view->render('theme::VaporChild/views/sitemap', ['posts' => $urls])
        );

        return $response->withHeader('Content-Type', 'text/xml');
    }
}
