<?php

declare(strict_types=1);

namespace Theme\VaporChild;

use App\Shared\Services\Registry;
use App\Shared\Services\SeoFactory;
use App\Shared\Services\Utils;
use Qubus\EventDispatcher\ActionFilter\Action;
use Qubus\EventDispatcher\ActionFilter\Filter;
use Qubus\Exception\Exception;
use ReflectionException;
use Theme\Vapor\VaporTheme;

use function App\Shared\Helpers\site_url;
use function App\Shared\Helpers\theme_root;
use function App\Shared\Helpers\theme_url;
use function basename;
use function get_class;
use function Qubus\Security\Helpers\t__;

use const JSON_PRETTY_PRINT;

final class VaporChildTheme extends VaporTheme
{
    /**
     * @inheritDoc
     * @throws ReflectionException|Exception
     */
    public function meta(): array
    {
        $theme = [
            'name' => t__(msgid: 'Vapor Child', domain: 'vapor-theme'),
            'id' => 'vapor-theme',
            'author' => 'Joshua Parker',
            'version' => '1.0.0',
            'description' => t__(msgid: 'Child theme of Vapor.', domain: 'vapor-theme'),
            'basename' => basename(__DIR__ . '/Vapor'),
            'path' => theme_root(__FILE__),
            'url' => theme_url('', __CLASS__),
            'themeUri' => 'https://github.com/getdevflow/vapor-theme',
            'authorUri' => 'https://nomadicjosh.com/',
            'className' => get_class($this),
            'screenshot' => theme_url('Vapor/screenshot.png'),
        ];

        Registry::getInstance()->set('vapor-theme', $theme);

        return $theme;
    }

    public function schema(): void
    {
        $schema = SeoFactory::schema(
            SeoFactory::thing('Website', [
                'url'          => site_url(),
                'logo'         => site_url('static/assets/img/auth/auth-logo.png'),
                'contactPoint' => SeoFactory::thing('ContactPoint', [
                    'telephone' => '+1-000-555-1212',
                    'contactType' => 'customer service'
                ])
            ])
        );

        echo '<script type="application/ld+json">' . "\n" . json_encode($schema, JSON_PRETTY_PRINT) . "\n" . '</script>';
    }

    public function metaTags(): void
    {
        $metaTags = SeoFactory::metaTags()
            ->title('Developer Blog')
            ->description('Blog featuring PHP tips and tricks.')
            ->meta('author', 'Joshua Parker')
            ->image('https://www.gravatar.com/avatar/cc144b35b739d6b30cea73a24469377c?s=160')
            ->canonical(site_url());

        echo $metaTags;
    }

    /**
     * @return void
     * @throws ReflectionException
     */
    public function frontendRender(): void
    {
        if (Utils::isAdmin()) {
            return;
        }

        Filter::getInstance()->addFilter(hook: 'theme_route', callback: function ($router) {
            $router->setDefaultNamespace('\\Theme\\VaporChild\\Controllers');
            $router->get('/', 'VaporThemeController@index');
            $router->get('/sitemap/', 'VaporThemeController@sitemap');
            $router->get('/{contentSlug}/', 'VaporThemeController@single');
        }, priority: 5);

        Action::getInstance()->addAction('cms_head', [$this, 'metaTags']);
        Action::getInstance()->addAction('cms_head', [$this, 'schema']);
    }
}
